from django.db import models
from django.contrib.auth.models import User
import sys
from datetime import datetime, date

User._meta.get_field("email")._unique = True
User._meta.get_field("email").blank = False
User._meta.get_field("email").null = False

class Plasmid(models.Model):
    name = models.CharField(max_length = 100)
    sequence = models.TextField(max_length = sys.maxsize)

class PlasmidPart(models.Model):
    name = models.CharField(max_length = 100)
    start_position = models.IntegerField()
    end_position = models.IntegerField()
    strand = models.CharField(max_length = 1)
    color = models.CharField(max_length = 100)

    plasmids = models.ManyToManyField(Plasmid)

class Batch(models.Model):
    num_fragments = models.IntegerField(default = 0)
    status = models.CharField(max_length=1000, default = "Unprocessed")

class Order(models.Model):
    name = models.CharField(max_length = 50)

    sequence = models.TextField(max_length = sys.maxsize)

    bacterial_selection = models.CharField(max_length = 100, blank = True, null = True)
    originReplication = models.CharField(max_length = 100, blank = True, null = True)
    organism = models.CharField(max_length = 100, blank = True, null = True)

    status = models.CharField(max_length = 10000, blank = True, null = True)

    created_at = models.DateField(auto_now_add = True, blank = True)
    updated_at = models.DateField(auto_now = True, blank = True)

    user = models.ForeignKey(User, blank = True, on_delete = models.CASCADE)
    batch = models.ForeignKey(Batch, blank = True, null = True, on_delete = models.CASCADE)

    accepted_quote = models.FloatField(blank = True, null = True)

class Feature(models.Model):
    start_position = models.IntegerField()
    end_position = models.IntegerField()
    name = models.CharField(max_length = 100)

    forward = models.BooleanField(default = True)

    order = models.ForeignKey(Order, on_delete = models.CASCADE)
    plasmid_part = models.ForeignKey(PlasmidPart, on_delete = models.CASCADE, blank = True, null = True)

    color = models.CharField(max_length = 100)

    has_custom_sequence = models.BooleanField(default = False)
    template_plasmid = models.CharField(max_length = 1000)
    sequence = models.TextField(max_length = sys.maxsize, blank = True, null = True)

class UserPersonalInfo(models.Model):
    user = models.OneToOneField(User, on_delete = models.CASCADE)

    first_name = models.CharField(max_length = 100)
    last_name = models.CharField(max_length = 100)
    affiliation = models.CharField(max_length = 100)

    verified = models.BooleanField(default = False)
    verification_token = models.CharField(max_length = 250)

    requested_technician_status = models.BooleanField(default = False)

class DNAFiles(models.Model):
    name = models.CharField(max_length = 100)
    upload = models.FileField(upload_to = "iBioFabBackend/dna_uploads")

class Notification(models.Model):
    user = models.ForeignKey(User, on_delete = models.CASCADE)

    header = models.CharField(max_length = 150)
    body = models.CharField(max_length = 10000)

    response_type = models.TextField(max_length = sys.maxsize)

    has_read = models.BooleanField(default = False)

    created_at = models.DateField(auto_now_add = True, blank = True)
    updated_at = models.DateField(auto_now = True, blank = True)

UserPersonalInfo._meta.get_field("first_name").blank = False
UserPersonalInfo._meta.get_field("first_name").null = False

UserPersonalInfo._meta.get_field("last_name").blank = False
UserPersonalInfo._meta.get_field("last_name").null = False

UserPersonalInfo._meta.get_field("affiliation").blank = False
UserPersonalInfo._meta.get_field("affiliation").null = False
