"""iBioFabBackend URL Configuration

The `urlpatterns` list routes URLs to views. For more information please see:
    https://docs.djangoproject.com/en/3.1/topics/http/urls/
Examples:
Function views
    1. Add an import:  from my_app import views
    2. Add a URL to urlpatterns:  path('', views.home, name='home')
Class-based views
    1. Add an import:  from other_app.views import Home
    2. Add a URL to urlpatterns:  path('', Home.as_view(), name='home')
Including another URLconf
    1. Import the include() function: from django.urls import include, path
    2. Add a URL to urlpatterns:  path('blog/', include('blog.urls'))
"""
from iBioFabBackend.views.batch_views import process_unprocessed_batches
from iBioFabBackend.views.order_views import copy_order
from django.contrib import admin
from django.urls import path, include
from django.conf.urls import url

from rest_framework.urlpatterns import format_suffix_patterns
from rest_framework.authtoken.views import obtain_auth_token

from . import views
from .views import RegisterAPI, LoginAPI, VerifyAPI, ProjectDesignDNAUploadView

from knox import views as knox_views

urlpatterns = [
    path('admin/', admin.site.urls),

    path("search_alignment_in_inventory/", views.search_alignment_in_inventory, name = "search_alignment_in_inventory"),
    path("search_inventory_by_name/", views.search_inventory_by_name, name = "search_inventory_by_name"),
    path("search_general_by_alignment_parts_and_plasmid_name/", views.search_general_by_alignment_parts_and_plasmid_name, name = "search_general_by_alignment_parts_and_plasmid_name"),
    path("project_design_dna_upload_view/", ProjectDesignDNAUploadView.as_view(), name = "project_design_dna_upload_view"),
    path("get_all_annotations/", views.obtain_all_parts, name = "get_all_annotations"),
    path("get_all_plasmids/", views.obtain_all_plasmids, name = "get_all_plasmids"),

    path("create_order/", views.create_order, name = "create_order"),
    path("edit_order/", views.update_order, name = "edit_order"),
    path("submit_order/", views.submit_order, name = "submit_order"),
    path("copy_order/", views.copy_order, name = "copy_order"),
    path("get_order/", views.get_order, name = "get_order"),
    path("update_order_name/", views.update_order_name, name = "update_order_name"),
    path("delete_order/", views.delete_order, name = "delete_order"),
    path("get_order_guides_and_primers/", views.get_order_guides_and_primers, name = "get_order_guides_and_primers"),

    # User login, logout, and registration endpoints
    path("api/login/", LoginAPI.as_view(), name = "login"),
    path("api/logout/", knox_views.LogoutView.as_view(), name = "logout"),
    path("api/register/", RegisterAPI.as_view(), name = 'register'),
    path("api/verify_token/", VerifyAPI.as_view(), name = "verify_token"),
    path("api/request_password_change/", views.request_password_change, name = "request_password_change"),
    path("api/submit_password_change/", views.submit_password_change, name = "submit_password_change"),

    # Searching backbones
    path("search_backbone/", views.search_backbone, name = "search_backbone"),

    # Translation
    path("get_translation/", views.obtain_translation, name = "get_translation"),

    path("obtain_user_information/", views.get_user_information, name = "obtain_user_information"),

    # Technician Related APIs
    path("process_batches/", views.process_unprocessed_batches, name = "process_batches"),

    # Notification API urls

    path("load_notifications_for_user/", views.load_notifications_for_user, name = "load_notifications_for_user"),
    path("send_quote_from_technician/", views.send_quote_from_technician, name = "send_quote_from_technician")



]
