from django.shortcuts import render, get_object_or_404
from django.contrib.auth import login, logout
from django.core.mail import send_mail
from django.conf import settings as conf_settings
from django.contrib.auth.models import User, Group

from rest_framework.decorators import api_view, permission_classes
from rest_framework.response import Response
from rest_framework import generics, permissions
from rest_framework.permissions import IsAuthenticated
from rest_framework.permissions import AllowAny
from rest_framework.authtoken.serializers import AuthTokenSerializer

from rest_framework.parsers import FileUploadParser
from rest_framework.views import APIView

from knox.models import AuthToken
from knox.views import LoginView as KnoxLoginView
from knox import views as knox_views
from ..serializers import UserSerializer, RegisterSerializer
from knox.auth import TokenAuthentication

from django.http import JsonResponse

import iBioFabBackend.cooked_up_libraries.flexible_alignment_search as fas
import iBioFabBackend.cooked_up_libraries.snapgene_annotation_parser as sap
import iBioFabBackend.cooked_up_libraries.api_helper_methods as ahm
import iBioFabBackend.cooked_up_libraries.plasmids_and_parts_updater as ppu

from snapgene_reader import snapgene_file_to_dict
from Bio import SeqIO
from Bio.Seq import Seq

import re
import tempfile

from ..models import Plasmid, PlasmidPart, Order, Feature, UserPersonalInfo, DNAFiles, Notification
from ..documents import PlasmidDocument, PlasmidPartDocument

import random
from io import BytesIO

import json
import os

from django.utils.timesince import timesince

import pandas as pd
import numpy as np

import os
import shutil

@api_view(["POST"])
@permission_classes((AllowAny, ))
def obtain_all_plasmids(request):
    if request.method == "POST":
        plasmids = []

        for plasmid in Plasmid.objects.all():
            plasmid_info = {
                "name": plasmid.name,
                "sequence": plasmid.sequence
            }

            parts_for_plasmid = []
            for plasmid_part_iter in plasmid.plasmidpart_set.all():
                start_position = plasmid_part_iter.start_position
                end_position = plasmid_part_iter.end_position


                if start_position < end_position:
                    part_sequence = plasmid.sequence[start_position - 1: end_position]
                else:
                    part_sequence = plasmid.sequence[end_position - 1: ] + plasmid.sequence[0: start_position - 1]

                parts_for_plasmid.append({
                    "id": plasmid_part_iter.id,
                    "name": plasmid_part_iter.name,
                    "start": plasmid_part_iter.start_position,
                    "end": plasmid_part_iter.end_position,
                    "color": plasmid_part_iter.color,
                    "sequence": part_sequence
                })

            plasmid_info["parts"] = parts_for_plasmid
            plasmids.append(plasmid_info)

        return JsonResponse({
            "plasmids": plasmids
        })

@api_view(["POST"])
@permission_classes((AllowAny, ))
def obtain_all_parts(request):
    if request.method == "POST":
        parts = PlasmidPart.objects.order_by("name")

        response = []
        names = []

        for part in parts:
            if part.name not in names:
                names.append(part.name)
                
                associated_plasmid_info = []
                first_plasmid = True
                part_sequence = ""
                plasmid_id_of_part_sequence = 0

                for plasmid in part.plasmids.all():
                    plasmid_info = {
                        "name": plasmid.name,
                        "sequence": plasmid.sequence
                    }

                    parts_for_plasmid = []
                    for plasmid_part_iter in plasmid.plasmidpart_set.all():
                        parts_for_plasmid.append({
                            "id": plasmid_part_iter.id,
                            "name": plasmid_part_iter.name,
                            "start": plasmid_part_iter.start_position,
                            "end": plasmid_part_iter.end_position,
                            "color": plasmid_part_iter.color
                        })

                        if first_plasmid:
                            if part.id == plasmid_part_iter.id:
                                start_position = part.start_position
                                end_position = part.end_position


                                if start_position < end_position:
                                    part_sequence = plasmid.sequence[start_position - 1: end_position]
                                else:
                                    part_sequence = plasmid.sequence[end_position - 1: ] + plasmid.sequence[0: start_position - 1]

                                plasmid_id_of_part_sequence = plasmid.id

                    first_plasmid = False

                    plasmid_info["parts"] = parts_for_plasmid

                    associated_plasmid_info.append(plasmid_info)

                response.append({
                    "id": part.id,
                    "name": part.name,
                    "start": part.start_position,
                    "end": part.end_position,
                    "color": part.color,
                    "plasmids": associated_plasmid_info,
                    "part_sequence": part_sequence,
                    "part_sequence_associated_plasmid_id": plasmid_id_of_part_sequence
                })

        return JsonResponse({
            "parts": response
        })