from django.shortcuts import render, get_object_or_404
from django.contrib.auth import login, logout
from django.core.mail import send_mail
from django.conf import settings as conf_settings
from django.contrib.auth.models import User, Group

from rest_framework.decorators import api_view, permission_classes
from rest_framework.response import Response
from rest_framework import generics, permissions
from rest_framework.permissions import IsAuthenticated
from rest_framework.permissions import AllowAny
from rest_framework.authtoken.serializers import AuthTokenSerializer

from rest_framework.parsers import FileUploadParser
from rest_framework.views import APIView

from knox.models import AuthToken
from knox.views import LoginView as KnoxLoginView
from knox import views as knox_views
from ..serializers import UserSerializer, RegisterSerializer
from knox.auth import TokenAuthentication

from django.http import JsonResponse

import iBioFabBackend.cooked_up_libraries.flexible_alignment_search as fas
import iBioFabBackend.cooked_up_libraries.snapgene_annotation_parser as sap
import iBioFabBackend.cooked_up_libraries.api_helper_methods as ahm
import iBioFabBackend.cooked_up_libraries.plasmids_and_parts_updater as ppu

from snapgene_reader import snapgene_file_to_dict
from Bio import SeqIO
from Bio.Seq import Seq

import re
import tempfile

from ..models import Plasmid, PlasmidPart, Order, Feature, UserPersonalInfo, DNAFiles, Notification
from ..documents import PlasmidDocument, PlasmidPartDocument

import random
from io import BytesIO
from iBioFabBackend.tasks import get_necessary_information_to_build_plasmid_orders as obtain_production_information

import json
import os

from django.utils.timesince import timesince

import pandas as pd
import numpy as np

import os
import shutil

############ The APIs in this section are concerning the creation and editing and submission of projects/orders
@api_view(["POST"])
@permission_classes((IsAuthenticated, ))
def create_order(request):
    if request.method == "POST":
        new_order = Order(user = request.user, name = "Project #%s" % (request.user.order_set.all().count() + 1), status = "User Construction")
        new_order.save()

        return JsonResponse({
            "success": {
                "order_id": new_order.id,
                "name": new_order.name
            }
        })

@api_view(["PATCH"])
@permission_classes((IsAuthenticated, ))
def update_order(request):
    if request.method == "PATCH":
        if ahm.has_all_fields_present(request, ["order_id"]):
            updated_order = Order.objects.get(pk = request.data["order_id"])

            if (updated_order.user.id == request.user.id and updated_order.status == "User Construction") or request.user.groups.filter(name = "Technician"):
                new_sequence = request.data["sequence"]
                updated_order.sequence = new_sequence

                updated_order.save()
                print(request.data)

                new_features = request.data["features"]

                updated_order.feature_set.all().delete() # Wipes out the current features associated with the order to replace with the
                # features given

                fragment_number = 1
                print(new_features)
                for new_feature in new_features:
                    if new_feature["custom"] == False:
                        new_feature_object = Feature(
                            start_position = new_feature["start"],
                            end_position = new_feature["end"],
                            name = new_feature["name"],
                            order = updated_order,
                            color = new_feature["color"],
                            plasmid_part = PlasmidPart.objects.get(pk = new_feature["id"]),
                            template_plasmid = PlasmidPart.objects.get(pk = new_feature["id"]).plasmids.all()[0].name,
                            forward = new_feature["forward"]
                        )

                        new_feature_object.save()
                    else:
                        if "template_plasmid" in new_feature:
                            new_feature_object = Feature(
                                start_position = new_feature["start"],
                                end_position = new_feature["end"],
                                name = new_feature["name"],
                                color = new_feature["color"],
                                order = updated_order,
                                has_custom_sequence = True,
                                sequence = new_sequence[new_feature["start"]: new_feature["end"] + 1],
                                template_plasmid = new_feature["template_plasmid"],
                                forward = new_feature["forward"]
                            )
                        else:
                            new_feature_object = Feature(
                                start_position = new_feature["start"],
                                end_position = new_feature["end"],
                                name = new_feature["name"],
                                color = new_feature["color"],
                                order = updated_order,
                                has_custom_sequence = True,
                                sequence = new_sequence[new_feature["start"]: new_feature["end"] + 1],
                                forward = new_feature["forward"]
                            )

                        new_feature_object.save()

                    fragment_number += 1

                # obtain_production_information.delay(new_order.id, order_dict)

                return JsonResponse({
                    "success": "Order successfully updated"
                })
            else:
                return JsonResponse({
                    "error": "Permission denied"
                })
        else:
            return JsonResponse({
                "error": "Not all fields present"
            })

@api_view(["POST"])
@permission_classes((IsAuthenticated, ))
def update_order_name(request):
    if request.method == "POST":
        if ahm.has_all_fields_present(request, ["order_id", "new_order_name"]):
            order = Order.objects.get(pk = request.data["order_id"])

            if request.user.id == order.user.id or request.user.groups.filter(name = "Technician"):
                order.name = request.data["new_order_name"]
                order.save()

                return JsonResponse({
                    "success": "Project name sucessfully updated"
                })
            else:
                return JsonResponse({
                    "error": "Permission denied"
                })
        else:
            return JsonResponse({
                "error": "Missing fields"
            })

@api_view(["POST"])
@permission_classes((IsAuthenticated, ))
def copy_order(request):
    if request.method == "POST":
        if ahm.has_all_fields_present(request, ["order_id"]):
            order = Order.objects.get(pk = request.data["order_id"])

            if request.user.id == order.user.id or request.user.groups.filter(name = "Technician"):
                copied_order = Order(
                    name = "Copy of %s" % (order.name),
                    sequence = order.sequence,
                    bacterial_selection = order.bacterial_selection,
                    originReplication = order.originReplication,
                    organism = order.organism,
                    status = "User Construction",
                    user = request.user
                )

                copied_order.save()

                for feature in order.feature_set.all().order_by("start_position"):
                    if feature.has_custom_sequence:
                        copied_feature = Feature(
                            start_position = feature.start_position,
                            end_position = feature.end_position,
                            name = feature.name,
                            color = feature.color,
                            forward = feature.forward,
                            order = copied_order,
                            has_custom_sequence = True,
                            template_plasmid = feature.template_plasmid,
                            sequence = feature.sequence
                        )

                        copied_feature.save()
                    else:
                        copied_feature = Feature(
                            start_position = feature.start_position,
                            end_position = feature.end_position,
                            name = feature.name,
                            color = feature.color,
                            forward = feature.forward,
                            order = copied_order,
                            has_custom_sequence = False,
                            template_plasmid = feature.template_plasmid,
                            sequence = feature.sequence,
                            plasmid_part = feature.plasmid_part
                        )

                        copied_feature.save()
                
                return JsonResponse({
                    "status": "Success",
                    "copied_order_id": copied_order.id
                })
            else:
                return JsonResponse({
                    "error": "Permission denied"
                })
        else:
            return JsonResponse({
                "error": "Missing fields present"
            })

@api_view(["POST"])
@permission_classes((IsAuthenticated, ))
def get_order(request):
    if request.method == "POST":
        if ahm.has_all_fields_present(request, ["order_id"]):
            order = Order.objects.get(pk = request.data["order_id"])

            if request.user.id == order.user.id or request.user.groups.filter(name = "Technician"):
                response = {
                    "order_id": order.id,
                    "name": order.name,
                    "sequence": order.sequence,
                    "backbone": {
                        "bacterialSelection": order.bacterial_selection,
                        "originReplication": order.originReplication,
                        "organism": order.organism
                    }
                }

                features = []

                for feature in order.feature_set.all().order_by("start_position"):
                    if feature.has_custom_sequence:
                        features.append({
                            "id": feature.id,
                            "start": feature.start_position,
                            "end": feature.end_position,
                            "name": feature.name,
                            "color": feature.color,
                            "custom": True,
                            "forward": feature.forward
                        })
                    else:
                        features.append({
                            "id": feature.plasmid_part.id,
                            "start": feature.start_position,
                            "end": feature.end_position,
                            "name": feature.name,
                            "color": feature.color,
                            "custom": False,
                            "forward": feature.forward
                        })

                response["features"] = features

                return JsonResponse(response)
            else:
                return JsonResponse({
                    "error": "Permission denied"
                })
        else:
            return JsonResponse({
                "error": "Missing fields"
            })

@api_view(['POST'])
@permission_classes((IsAuthenticated, ))
def submit_order(request):
    if request.method == "POST":
        if ahm.has_all_fields_present(request, ["order_id"]):
            order = Order.objects.get(pk = request.data["order_id"])

            if order.user.id == request.user.id or request.user.groups.filter(name = "Technician"):
                order_dict = ahm.get_order_information_for_script_processing(order)

                print(order_dict)

                obtain_production_information.delay([order.id], [order_dict])

                order.status = "Quote Request"
                order.save()

                return JsonResponse({
                    "output": "Order successfully submitted for review."
                })
            else:
                return JsonResponse({
                    "error": "Permission denied"
                })
        else:
            return JsonResponse({
                "error": "Missing fields"
            })

@api_view(["POST"])
@permission_classes((IsAuthenticated, ))
def delete_order(request):
    if request.method == "POST":
        user = request.user
        order = Order.objects.get(pk = request.data["order_id"])

        if order.user.id == request.user.id or request.user.groups.filter(name = "Technician"):
            order.delete()

            return JsonResponse({
                "success": "Order Deleted"
            })
        else:
            return JsonResponse({
                "error": "Permission Denied"
            })

@api_view(["POST"])
@permission_classes((IsAuthenticated, ))
def get_order_guides_and_primers(request):
    if request.method == "POST":
        if ahm.has_all_fields_present(request, ["order_id"]):
            user = request.user
            order = Order.objects.get(pk = request.data["order_id"])

            if order.user.id == request.user.id or request.user.groups.filter(name = "Technician"):
                if os.path.isdir("iBioFabBackend/Order_Useful_Information/%s" % (order.id)):
                    relevant_directory = os.path.isdir("iBioFabBackend/Order_Useful_Information/%s" % (order.id))

                    if len(os.listdir(relevant_directory)) == 4:
                        zip_file_location = "iBioFabBackend/Order_Useful_Information/zip_file_outputs/%s.zip" % (order.id)

                        shutil.make_archive(relevant_directory, "zip", relevant_directory)

                        zip_file = open(zip_file_location, "rb")

                        return JsonResponse({
                            "status": "Success",
                            "blob": zip_file.read()
                        })
                    else:
                        return JsonResponse({
                            "status": "Results NA"
                        })
                else:
                    return JsonResponse({
                        "status": "Results NA"
                    })
            else:
                return JsonResponse({
                    "error": "Unauthorized Access"
                })
        else:
            return JsonResponse({
                "error": "Missing fields"
            })