from django.shortcuts import render, get_object_or_404
from django.contrib.auth import login, logout
from django.core.mail import send_mail
from django.conf import settings as conf_settings
from django.contrib.auth.models import User, Group

from rest_framework.decorators import api_view, permission_classes
from rest_framework.response import Response
from rest_framework import generics, permissions
from rest_framework.permissions import IsAuthenticated
from rest_framework.permissions import AllowAny
from rest_framework.authtoken.serializers import AuthTokenSerializer

from rest_framework.parsers import FileUploadParser
from rest_framework.views import APIView

from knox.models import AuthToken
from knox.views import LoginView as KnoxLoginView
from knox import views as knox_views
from ..serializers import UserSerializer, RegisterSerializer
from knox.auth import TokenAuthentication

from django.http import JsonResponse

import iBioFabBackend.cooked_up_libraries.flexible_alignment_search as fas
import iBioFabBackend.cooked_up_libraries.snapgene_annotation_parser as sap
import iBioFabBackend.cooked_up_libraries.api_helper_methods as ahm
import iBioFabBackend.cooked_up_libraries.plasmids_and_parts_updater as ppu

from snapgene_reader import snapgene_file_to_dict
from Bio import SeqIO
from Bio.Seq import Seq

import re
import tempfile

from ..models import Plasmid, PlasmidPart, Order, Feature, UserPersonalInfo, DNAFiles, Notification
from ..documents import PlasmidDocument, PlasmidPartDocument

import random
from io import BytesIO

import json
import os

from django.utils.timesince import timesince

import pandas as pd
import numpy as np

import os
import shutil

@api_view(['POST'])
@permission_classes((AllowAny, ))
def search_alignment_in_inventory(request):
    if request.method == "POST":
        sequences = request.data["sequences"]

        alignment_dict = fas.conduct_alignment_with_sequence_list(sequences, "iBioFabBackend/temp", "test_database.fasta")

        return JsonResponse(alignment_dict)

@api_view(['POST'])
@permission_classes((AllowAny, ))
def search_inventory_by_name(request):
    if request.method == "POST":
        sequence_name = request.data["plasmid_name"]

        sequence_snapgene_dict = snapgene_file_to_dict("example_inventory_plasmids/%s.dna" % (sequence_name))
        sequence = sequence_snapgene_dict["seq"]

        annotations_raw = sequence_snapgene_dict["features"]

        annotation_output = {"num_annotations": len(annotations_raw)}
        features = []

        annotation_number = 1
        for annotation in annotations_raw:
            start = annotation["start"]
            end = annotation["end"]

            type = annotation["type"]
            name = annotation["name"]

            annotation_dict = {"name": name, "type": type, "start_position": start, "end_position": end}
            annotation_output[annotation_number] = annotation_dict

            features.append({"id": annotation_number, name: "%s (%s)" % (name, type), "start": start, "end": end})

            annotation_number += 1

        fasta_sequences = SeqIO.parse(open("test_database/test_database.fasta"), "fasta")
        for seq in fasta_sequences:
            if seq.id == request.data["plasmid_name"]:
                response_string = ""
                for i in range(len(seq)):
                    response_string += seq[i]

                response = {"sequence": response_string, "annotations": annotation_output, "features": features}
                return JsonResponse(response)

@api_view(['POST'])
@permission_classes((AllowAny, ))
def search_general_by_alignment_parts_and_plasmid_name(request):
    if request.method == "POST":
        if ahm.has_all_fields_present(request, ["search"]):
            search_string = request.data["search"]

            dna_regex_string = "^[AaTtCcGg]*$"

            is_dna_string = True
            if re.match(dna_regex_string, search_string) == None:
                is_dna_string = False

            alignments = {}
            parts = []
            plasmids = []

            if is_dna_string:
                alignments = fas.conduct_alignment_with_sequence_list([search_string], "iBioFabBackend/temp", "test_database.fasta")

            plasmid_name_search = PlasmidDocument.search().filter("match", name = search_string)
            plasmid_parts_search = PlasmidPartDocument.search().filter("match", name = search_string)

            plasmid_name_search = ppu.search_plasmid_name(search_string)
            plasmid_parts_search = ppu.search_plasmid_annotation(search_string)

            for plasmid_name_hits in plasmid_name_search:
                plasmid_information = {"name": plasmid_name_hits["name"], "sequence": plasmid_name_hits["sequence"]}

                plasmid = Plasmid.objects.get(id = plasmid_name_hits["sql_id"])
                parts_for_plasmid = []
                for plasmid_part in plasmid.plasmidpart_set.all():
                    parts_for_plasmid.append({
                        "id": plasmid_part.id,
                        "name": plasmid_part.name,
                        "start": plasmid_part.start_position,
                        "end": plasmid_part.end_position,
                        "color": plasmid_part.color
                    })

                plasmid_information["parts"] = parts_for_plasmid
                plasmids.append(plasmid_information)

            for plasmid_parts_name_hits in plasmid_parts_search:
                plasmid_part = PlasmidPart.objects.get(id = plasmid_parts_name_hits["sql_id"])

                associated_plasmid_info = []
                first_plasmid = True
                part_sequence = ""
                plasmid_id_of_part_sequence = 0

                for plasmid in plasmid_part.plasmids.all():
                    plasmid_info = {
                        "name": plasmid.name,
                        "sequence": plasmid.sequence
                    }

                    parts_for_plasmid = []
                    for plasmid_part_iter in plasmid.plasmidpart_set.all():
                        parts_for_plasmid.append({
                            "id": plasmid_part_iter.id,
                            "name": plasmid_part_iter.name,
                            "start": plasmid_part_iter.start_position,
                            "end": plasmid_part_iter.end_position,
                            "color": plasmid_part_iter.color
                        })

                        if first_plasmid:
                            if plasmid_part.id == plasmid_part_iter.id:
                                start_position = plasmid_part.start_position
                                end_position = plasmid_part.end_position


                                if start_position < end_position:
                                    part_sequence = plasmid.sequence[start_position - 1: end_position]
                                else:
                                    part_sequence = plasmid.sequence[end_position - 1: ] + plasmid.sequence[0: start_position - 1]

                                plasmid_id_of_part_sequence = plasmid.id

                    first_plasmid = False

                    plasmid_info["parts"] = parts_for_plasmid

                    associated_plasmid_info.append(plasmid_info)

                parts.append({
                    "id": plasmid_part.id,
                    "name": plasmid_part.name,
                    "start": plasmid_part.start_position,
                    "end": plasmid_part.end_position,
                    "color": plasmid_part.color,
                    "plasmids": associated_plasmid_info,
                    "part_sequence": part_sequence,
                    "part_sequence_associated_plasmid_id": plasmid_id_of_part_sequence
                })


            response = {"alignments": alignments, "parts": parts, "plasmids": plasmids}

            json_formatted_str = json.dumps(response, indent = 4)

            return JsonResponse(response)
        else:
            return JsonResponse({
                "error": "Not all fields present"
            })

@api_view(["POST"])
@permission_classes((AllowAny, ))
def search_backbone(request):
    if request.method == "POST":
        backbone_df = pd.read_excel("Biodata/Backbones.xlsx")

        filtered_df = backbone_df

        if request.data != {}:
            for key in request.data.keys():
                filtered_df = filtered_df.loc[filtered_df[key].isin([request.data[key]])]

        response_json = {
            "plasmids": [],
            "species": [],
            "vector_type": [],
            "bacterial_resistance": [],
            "tags": []
        }

        plasmids = []

        species = []
        vector_type = []
        bacterial_resistance = []
        tags = []

        print(filtered_df)

        for index, row in filtered_df.iterrows():
            plasmid_model = Plasmid.objects.get(name = row["plasmid_name"])

            output_tag = row["tags"]

            if pd.isna(output_tag):
                output_tag = None

            output_notes = row["notes"]

            if pd.isna(output_notes):
                output_notes = None

            # if np.isnan(output_tag):
            #     output_tag = None

            plasmid = {
                "name": row["plasmid_name"],
                "species": row["species"],
                "vector_type": row["vector_type"],
                "bacterial_resistance": row["bacterial_resistance"],
                "tags": output_tag,
                "notes": output_notes,
                "sequence": plasmid_model.sequence
            }

            features = []

            for plasmid_part in plasmid_model.plasmidpart_set.all():
                features.append({
                    "name": plasmid_part.name,
                    "start_position": plasmid_part.start_position,
                    "end_position": plasmid_part.end_position,
                    "strand": plasmid_part.strand,
                    "color": plasmid_part.color
                })

            plasmid["features"] = features

            plasmids.append(plasmid)

            if row["species"] not in species:
                if not pd.isna(row["species"]):
                    species.append(row["species"])

            if row["vector_type"] not in vector_type:
                if not pd.isna(row["vector_type"]):
                    vector_type.append(row["vector_type"])

            if row["bacterial_resistance"] not in bacterial_resistance:
                if not pd.isna(row["bacterial_resistance"]):
                    bacterial_resistance.append(row["bacterial_resistance"])

            if row["tags"] not in tags:
                if not pd.isna(row["tags"]):
                    tags.append(row["tags"])

        response_json["species"] = species
        response_json["vector_type"] = vector_type
        response_json["bacterial_resistance"] = bacterial_resistance
        response_json["tags"] = tags

        response_json["plasmids"] = plasmids



        return JsonResponse(response_json)