from django.shortcuts import render, get_object_or_404
from django.contrib.auth import login, logout
from django.core.mail import send_mail
from django.conf import settings as conf_settings
from django.contrib.auth.models import User, Group

from rest_framework.decorators import api_view, permission_classes
from rest_framework.response import Response
from rest_framework import generics, permissions
from rest_framework.permissions import IsAuthenticated
from rest_framework.permissions import AllowAny
from rest_framework.authtoken.serializers import AuthTokenSerializer

from rest_framework.parsers import FileUploadParser
from rest_framework.views import APIView

from knox.models import AuthToken
from knox.views import LoginView as KnoxLoginView
from knox import views as knox_views
from ..serializers import UserSerializer, RegisterSerializer
from knox.auth import TokenAuthentication

from django.http import JsonResponse

import iBioFabBackend.cooked_up_libraries.flexible_alignment_search as fas
import iBioFabBackend.cooked_up_libraries.snapgene_annotation_parser as sap
import iBioFabBackend.cooked_up_libraries.api_helper_methods as ahm
import iBioFabBackend.cooked_up_libraries.plasmids_and_parts_updater as ppu

from snapgene_reader import snapgene_file_to_dict
from Bio import SeqIO
from Bio.Seq import Seq

import re
import tempfile

from ..models import Plasmid, PlasmidPart, Order, Feature, UserPersonalInfo, DNAFiles, Notification
from ..documents import PlasmidDocument, PlasmidPartDocument

import random
from io import BytesIO

import json
import os

from django.utils.timesince import timesince

import pandas as pd
import numpy as np

import os
import shutil

@api_view(["POST"])
@permission_classes((IsAuthenticated, ))
def send_quote_from_technician(request):
    if request.method == "POST":
        user = request.user

        if user.groups.filter(name = "Technician"):
            order_id = request.data["order_id"]

            attributed_quote = request.data["quote"]

            if attributed_quote.replace(".", "", 1).isdigit():
                attributed_quote_split = attributed_quote.split(".")

                valid_quote_text = True

                if len(attributed_quote_split) == 2:
                    if len(attributed_quote_split[1]) > 2:
                        valid_quote_text = False

                if valid_quote_text:
                    order = Order.objects.get(pk = order_id)

                    order.accepted_quote = float(attributed_quote)
                    order.status = "Quote Sent"
                    order.save()

                    user_who_made_order = order.user

                    email_subject = "Your order named %s for iBioFab has been quoted." % (order.name)
                    email_body = "iBioFab has quoted $%s for the construction and delivery of your custom plasmid" % (order.accepted_quote)

                    new_notification = Notification(
                        user = user_who_made_order,
                        header = email_subject,
                        body = email_body,
                        response_type = "Customer Plasmid Quote"
                    )

                    send_mail(
                        email_subject,
                        email_body,
                        EMAIL_HOST_USER,
                        [user_who_made_order.email]
                    )

                    notification_id = request.data["notification_id"]

                    notification = Notification.objects.get(pk = notification_id)
                    notification.has_read = True
                    notification.save()

                    return JsonResponse({
                        "success": "Quote sent to customer"
                    })
                else:
                    return JsonResponse({
                        "error": "Quote must have at most two decimal places"
                    })
            else:
                return JsonResponse({
                    "error": "Quote must be a number with at most two decimal places"
                })
        else:
            return JsonResponse({
                "error": "Permission denied"
            })