from django.shortcuts import render, get_object_or_404
from django.contrib.auth import login, logout
from django.core.mail import send_mail
from django.conf import settings as conf_settings
from django.contrib.auth.models import User, Group

from rest_framework.decorators import api_view, permission_classes
from rest_framework.response import Response
from rest_framework import generics, permissions
from rest_framework.permissions import IsAuthenticated
from rest_framework.permissions import AllowAny
from rest_framework.authtoken.serializers import AuthTokenSerializer

from rest_framework.parsers import FileUploadParser
from rest_framework.views import APIView

from knox.models import AuthToken
from knox.views import LoginView as KnoxLoginView
from knox import views as knox_views
from ..serializers import UserSerializer, RegisterSerializer
from knox.auth import TokenAuthentication

from django.http import JsonResponse

import iBioFabBackend.cooked_up_libraries.flexible_alignment_search as fas
import iBioFabBackend.cooked_up_libraries.snapgene_annotation_parser as sap
import iBioFabBackend.cooked_up_libraries.api_helper_methods as ahm
import iBioFabBackend.cooked_up_libraries.plasmids_and_parts_updater as ppu

from snapgene_reader import snapgene_file_to_dict
from Bio import SeqIO
from Bio.Seq import Seq

import re
import tempfile

from ..models import Plasmid, PlasmidPart, Order, Feature, UserPersonalInfo, DNAFiles, Notification
from ..documents import PlasmidDocument, PlasmidPartDocument

import random
from io import BytesIO

import json
import os

from django.utils.timesince import timesince

import pandas as pd
import numpy as np

import os
import shutil

# Parses DNA files
class ProjectDesignDNAUploadView(APIView):
    parser_class = (FileUploadParser, )
    permission_classes = [AllowAny]
    def post(self, request, format = None):
        if "file" not in request.data:
            return JsonResponse({
                "error": "File not uploaded"
            })

        dna_file = BytesIO(request.FILES['file'].read())
        new_dna_file_object = DNAFiles()
        new_dna_file_object.upload.save(str(request.data["file"]), dna_file, save = True)

        new_dna_file_abosolute_path = new_dna_file_object.upload.path

        annotation_parts = sap.obtain_annotations_from_snapgene_file(new_dna_file_abosolute_path)

        new_dna_file_object.upload.delete(save = True)
        new_dna_file_object.delete()

        return JsonResponse({
            "success": annotation_parts
        })